@@ -38,7 +38,7 @@ pub fn header<'a>(req: &'a HttpRequest, key: &str) -> Result<&'a str> {
 }
 
 fn modified_since_duration(req: &HttpRequest, datetime: DateTime<Utc>) -> Option<chrono::Duration> {
-    header(req, &http::header::IF_MODIFIED_SINCE.as_str()).ok()
+    header(req, http::header::IF_MODIFIED_SINCE.as_str()).ok()
         .and_then(|value| chrono::DateTime::parse_from_rfc2822(value).ok())
         .map(|value| value.signed_duration_since(datetime))
 }
@@ -109,7 +109,7 @@ pub async fn list_pkgs(
         let latest_built_at = SystemTime::from(latest_built_at);
         builder.set(http::header::LastModified(latest_built_at.into()));
     }
- 
+
     let mut pkgs = Vec::<PkgRelease>::new();
     for pkg in models::Package::list(connection.as_ref())? {
         if opt_filter(&pkg.name, query.name.as_deref()) {
@@ -177,7 +177,7 @@ fn get_worker_from_request(req: &HttpRequest, cfg: &Config, connection: &SqliteC
     } else {
         let worker = models::NewWorker::new(key.to_string(), ip, None);
         worker.insert(connection)?;
-        get_worker_from_request(req, &cfg, connection)
+        get_worker_from_request(req, cfg, connection)
     }
 }
 
@@ -202,7 +202,7 @@ pub async fn push_queue(
         debug!("found pkg: {:?}", pkg);
         let version = query.version.as_ref().unwrap_or(&pkg.version);
 
-        let item = models::NewQueued::new(pkg.id, version.to_string(), query.priority);
+        let item = models::NewQueued::new(pkg.id, version.to_string(), query.distro.to_string(), query.priority);
         debug!("adding to queue: {:?}", item);
         item.insert(connection.as_ref())?;
     }
@@ -214,7 +214,7 @@ pub async fn push_queue(
 pub async fn pop_queue(
     req: HttpRequest,
     cfg: web::Data<Config>,
-    _query: web::Json<WorkQuery>,
+    query: web::Json<WorkQuery>,
     pool: web::Data<Pool>,
 ) -> web::Result<impl Responder> {
     if auth::worker(&cfg, &req).is_err() {
@@ -226,7 +226,7 @@ pub async fn pop_queue(
     let mut worker = get_worker_from_request(&req, &cfg, connection.as_ref())?;
 
     models::Queued::free_stale_jobs(connection.as_ref())?;
-    let (resp, status) = if let Some(item) = models::Queued::pop_next(worker.id, connection.as_ref())? {
+    let (resp, status) = if let Some(item) = models::Queued::pop_next(worker.id, &query.supported_backends, connection.as_ref())? {
 
 
         // TODO: claim item correctly
@@ -300,19 +300,19 @@ pub async fn requeue_pkg(
         }
 
         debug!("pkg is going to be requeued: {:?} {:?}", pkg.name, pkg.version);
-        pkgs.push((pkg.id, pkg.version));
+        pkgs.push(pkg);
     }
 
     // TODO: use queue_batch after https://github.com/diesel-rs/diesel/pull/1884 is released
     // models::Queued::queue_batch(&pkgs, connection.as_ref())?;
-    for (id, version) in &pkgs {
-        let q = models::NewQueued::new(*id, version.to_string(), query.priority);
+    for pkg in &pkgs {
+        let q = models::NewQueued::new(pkg.id, pkg.version.to_string(), pkg.distro.to_string(), query.priority);
         q.insert(connection.as_ref()).ok();
     }
 
     if query.reset {
         let reset = pkgs.into_iter()
-            .map(|x| x.0)
+            .map(|x| x.id)
             .collect::<Vec<_>>();
         models::Package::reset_status_for_requeued_list(&reset, connection.as_ref())?;
     }
@@ -370,10 +370,11 @@ pub async fn report_build(
     let mut pkg = models::Package::get_id(item.package_id, connection.as_ref())?;
 
     let build = models::NewBuild::from_api(&report);
-    let build = build.insert(&connection.as_ref())?;
+    let build = build.insert(connection.as_ref())?;
     pkg.build_id = Some(build);
 
     pkg.has_diffoscope = report.rebuild.diffoscope.is_some();
+    pkg.has_attestation = report.rebuild.attestation.is_some();
 
     if report.rebuild.status == BuildStatus::Good {
         pkg.next_retry = None;
@@ -409,6 +410,30 @@ pub async fn get_build_log(
     Ok(resp)
 }
 
+#[get("/api/v0/builds/{id}/attestation")]
+pub async fn get_attestation(
+    id: web::Path<i32>,
+    pool: web::Data<Pool>,
+) -> web::Result<impl Responder> {
+    let connection = pool.get().map_err(Error::from)?;
+
+    let build = match models::Build::get_id(*id, connection.as_ref()) {
+        Ok(build) => build,
+        Err(_) => return Ok(not_found()),
+    };
+
+    if let Some(attestation) = build.attestation {
+        let resp = HttpResponse::Ok()
+            .content_type("application/json; charset=utf-8")
+            .header("X-Content-Type-Options", "nosniff")
+            .header("Content-Security-Policy", "default-src 'none'")
+            .body(attestation);
+        Ok(resp)
+    } else {
+        Ok(not_found())
+    }
+}
+
 #[get("/api/v0/builds/{id}/diffoscope")]
 pub async fn get_diffoscope(
     id: web::Path<i32>,
